//*****************************************************************************
//
// bdc-ui.c - User interface to control a RDK-BDC.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#include "../../../hw_ints.h"
#include "../../../hw_sysctl.h"
#include "../../../hw_types.h"
#include "../../../driverlib/interrupt.h"
#include "../../../driverlib/sysctl.h"
#include "../../../driverlib/systick.h"
#include "../../../grlib/grlib.h"
#include "../../../grlib/widget.h"
#include "../../../grlib/canvas.h"
#include "../can_proto.h"
#include "about.h"
#include "bdc-ui.h"
#include "buttons.h"
#include "can_comm.h"
#include "dev_list.h"
#include "help.h"
#include "menu.h"
#include "rit128x96x4.h"
#include "splash.h"
#include "update.h"
#include "voltage.h"

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Brushed DC User Interface (bdc-ui)</h1>
//!
//! This application provides a simple user interface for the Brushed DC Motor
//! Controller board, running on the EK-LM3S2965 board and communicating over
//! CAN.  In addition to running the motor, the motor status can be viewed, the
//! CAN network enumerated, and the motor controller's firmware can be updated.
//!
//! The direction buttons (left, right, up, and down) on the left side of the
//! EK-LM3S2965 are used to navigate through the user interface, and the select
//! button on the right side of the EK-LM3S2965 is used to select items.
//!
//! The user interface is divided into several panels; the top line of the
//! display always contains the name of the current panel.  By moving the
//! cursor to the top line and pressing select, a menu is displayed which will
//! allow a different panel to be displayed by pressing select again.
//!
//! The panels in the user interface will be individually discussed below.  At
//! startup, the Voltage Control Mode panel is displayed first.
//!
//!
//! <b>Voltage Control Mode</b>
//!
//! The voltage control mode panel allows the motor to be controlled by
//! directly selecting the output voltage.  The speed of the motor is directly
//! proportional to the voltage applied, and applying a ``negative'' voltage
//! (in other words, electronically reversing the power and ground connections)
//! will result in the motor spinning in the opposite direction.
//!
//! There are three parameters that can be adjusted on this panel; the ID,
//! voltage, and ramp rate.  The up and down buttons are used to select the
//! parameter to be modified, and the left and right buttons are used to adjust
//! the parameter's value.  The following parameters can be adjusted:
//!
//! - ID, which selects the motor controller to which commands are sent.  If
//!   the ID is changed while the motor is running, the motor will be stopped.
//!
//! - Voltage, which specifies the output voltage sent from the motor
//!   controller to the motor.  A positive voltage will result in voltage being
//!   applied to the white output terminal and ground being applied to the
//!   green output terminal, while a negative voltage will apply voltage to the
//!   green output terminal and ground to the white output terminal.
//!
//!   If the select button is pressed, changes to the output voltage will not
//!   be sent to the motor controller immediately (allowing the ramp to be
//!   used).  The text color of the voltage changes from white to black to
//!   indicate that a deferred update is active.  Pressing select again will
//!   send the final output voltage to the motor controller.
//!
//! - Ramp, which specifies the rate of change of the output voltage.  When set
//!   to ``none'', the output voltage will change immediately.  When set to a
//!   value, the output voltage is slowly changed from the current to to the
//!   target value at the specified rate.  This can be used to avoid browning
//!   out the power supply or to avoid over-torquing the motor on startup
//!   (for example preventing a loss of traction when a wheel is being driven).
//!
//! The bottom portion of the panel provides the current motor controller
//! status.
//!
//!
//! <b>Device List</b>
//!
//! This panel lists the motor controllers that reside on the CAN network.  All
//! 63 possible device IDs are listed, with those that are not present shown in
//! a dark gray and those that are present in a bright white.  By moving the
//! cursor to a particular ID and pressing the select button, a device ID
//! assignment will be performed.  The motor controller(s) will wait for five
//! seconds after an assignment request for its button to be pressed,
//! indicating that it should accept the device ID assignment.  So, for
//! example, if there are three motor controllers on a network, the following
//! sequence can be used to give them each unique IDs:
//!
//! - Move the cursor to number 1 and press select.  The LED on all three motor
//!   controllers will blink green to indicate that assignment mode is active.
//!
//! - Press the button on one of the motor controllers.  It will blink its LED
//!   yellow one time to indicate that it's ID is one.
//!
//! - Move the cursor to number 2 and press select.
//!
//! - Press the button on the second motor controller.  It will blink its LED
//!   yellow two times to indicate that it's ID is two.
//!
//! - Move the cursor to number 3 and press select.
//!
//! - Press the button on the third motor controller.  It will blink its LED
//!   yellow three times to indicate that it's ID is three.
//!
//! Once complete, this panel will then show that there are devices at IDs 1,
//! 2, and 3.
//!
//!
//! <b>Firmware Update</b>
//!
//! This panel allows the firmware on the motor controller to be updated over
//! the CAN network.  A firmware image for the motor controller is stored in
//! the flash of the EK-LM3S2965 board and used to update the motor controller.
//! The local copy of the motor controller firmware can be updated using the
//! UART boot loader protocol to transfer the image from a PC.  When the UART
//! ``update'' begins, this panel will automatically be displayed.
//!
//! The ID of the motor controller to be updated can be changed on this panel.
//! By using the local firmware image, multiple motor controllers can be
//! updated (one at a time) using this panel, without the need to redownload
//! from a PC each time.
//!
//! When not updating, the firmware version of the currently selected motor
//! controller will be displayed.  If there is no motor controller on the CAN
//! network with the current ID, the firmware version will be displayed as
//! ``---''.
//!
//! By pressing the select button when the ``Start'' button is highlighted, the
//! update of the motor controller firmware will commence.
//!
//! When the firmware is being transferred (either from the PC using the UART
//! or to the motor controller using the CAN network), the ID, firmware
//! version, and ``Start'' buttons will all be grayed out.  A progress bar will
//! appear below them to indicate what is happening and the how far it is
//! through the process.
//!
//!
//! <b>Help</b>
//!
//! This panel displays a condensed version of this application help text.  Use
//! the up and down buttons to scroll through the text.
//!
//!
//! <b>About</b>
//!
//! This panel simply displays the startup splash screen.
//
//*****************************************************************************

//*****************************************************************************
//
// A set of flags that indicate global status for the application.
//
//*****************************************************************************
unsigned long g_ulFlags;

//*****************************************************************************
//
// A widget that causes the entire display to be cleared.
//
//*****************************************************************************
static Canvas(g_sBackground, 0, 0, 0, &g_sRIT128x96x4Display, 0, 0, 128, 96,
              CANVAS_STYLE_FILL, ClrBlack, 0, 0, 0, 0, 0, 0);

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFileName, unsigned long ulLine)
{
}
#endif

//*****************************************************************************
//
// This function is called by SysTick once every millisecond.
//
//*****************************************************************************
void
SysTickIntHandler(void)
{
    unsigned char ucButtons;

    //
    // Set the flag that indicates that a timer tick has occurred.
    //
    HWREGBITW(&g_ulFlags, FLAG_TICK) = 1;

    //
    // Call the CAN periodic tick function.
    //
    CANTick();

    //
    // Call the push button periodic tick function.
    //
    ucButtons = ButtonsTick();

    //
    // Set the appropriate button press flags if the corresponding button was
    // pressed.
    //
    if(ucButtons & BUTTON_UP)
    {
        HWREGBITW(&g_ulFlags, FLAG_UP_PRESSED) = 1;
    }
    if(ucButtons & BUTTON_DOWN)
    {
        HWREGBITW(&g_ulFlags, FLAG_DOWN_PRESSED) = 1;
    }
    if(ucButtons & BUTTON_LEFT)
    {
        HWREGBITW(&g_ulFlags, FLAG_LEFT_PRESSED) = 1;
    }
    if(ucButtons & BUTTON_RIGHT)
    {
        HWREGBITW(&g_ulFlags, FLAG_RIGHT_PRESSED) = 1;
    }
    if(ucButtons & BUTTON_SELECT)
    {
        HWREGBITW(&g_ulFlags, FLAG_SELECT_PRESSED) = 1;
    }
}

//*****************************************************************************
//
// This function displays the splash screen.
//
//*****************************************************************************
static void
DisplaySplash(void)
{
    unsigned long ulIdx;
    tContext sContext;
    tRectangle sRect;

    //
    // Initialize a drawing context.
    //
    GrContextInit(&sContext, &g_sRIT128x96x4Display);

    //
    // Clear the screen.
    //
    sRect.sXMin = 0;
    sRect.sYMin = 0;
    sRect.sXMax = 127;
    sRect.sYMax = 95;
    GrContextForegroundSet(&sContext, ClrBlack);
    GrRectFill(&sContext, &sRect);

    //
    // Draw the splash screen image on the screen.
    //
    GrImageDraw(&sContext, g_pucSplashImage, 0, 10);

    //
    // Draw some text below the splash screen image.
    //
    GrContextForegroundSet(&sContext, ClrWhite);
    GrContextFontSet(&sContext, &g_sFontFixed6x8);
    GrStringDrawCentered(&sContext, "Brushed DC Motor", -1, 63, 73, 0);
    GrStringDrawCentered(&sContext, "Reference Design Kit", -1, 63, 81, 0);

    //
    // Flush the drawing operations to the screen.
    //
    GrFlush(&sContext);

    //
    // Delay for 5 seconds while the splash screen is displayed.
    //
    for(ulIdx = 0; ulIdx < 5000; ulIdx++)
    {
        HWREGBITW(&g_ulFlags, FLAG_TICK) = 0;
        while(HWREGBITW(&g_ulFlags, FLAG_TICK) == 0)
        {
        }
    }
}

//*****************************************************************************
//
// This function causes the display to be redrawn.
//
//*****************************************************************************
void
DisplayFlush(void)
{
    //
    // Send a paint message to the entire widget tree.
    //
    WidgetPaint(WIDGET_ROOT);

    //
    // Process any widget messages in the message queue.
    //
    WidgetMessageQueueProcess();

    //
    // Flush the drawing operations to the screen.
    //
    DpyFlush(&g_sRIT128x96x4Display);
}

//*****************************************************************************
//
// The main loop for the user interface.
//
//*****************************************************************************
int
main(void)
{
    unsigned long ulPanel;

    //
    // If running on Rev A2 silicon, turn the LDO voltage up to 2.75V.  This is
    // a workaround to allow the PLL to operate reliably.
    //
    if(REVISION_IS_A2)
    {
        SysCtlLDOSet(SYSCTL_LDO_2_75V);
    }

    //
    // Set the clocking to run at 50MHz from the PLL.
    //
    SysCtlClockSet(SYSCTL_SYSDIV_4 | SYSCTL_USE_PLL | SYSCTL_OSC_MAIN |
                   SYSCTL_XTAL_8MHZ);

    //
    // Set the priority of the interrupts.
    //
    IntPrioritySet(INT_CAN0, 0x00);
    IntPrioritySet(FAULT_SYSTICK, 0x20);

    //
    // Configure SysTick to generate an interrupt every millisecond.
    //
    SysTickPeriodSet(SysCtlClockGet() / 1000);
    SysTickIntEnable();
    SysTickEnable();

    //
    // Initialize the push button driver.
    //
    ButtonsInit();

    //
    // Initialize the CAN communication channel.
    //
    CANCommInit();

    //
    // Initialize the UART used to perform a "firmware update".
    //
    UpdateUARTInit();

    //
    // Initialize the display.
    //
    RIT128x96x4Init(1000000);

    //
    // Add the screen-clearing widget to the widget tree.  As the first widget
    // in the tree, this will always be drawn first, resulting in a blank
    // screen before anything else is drawn.
    //
    WidgetAdd(WIDGET_ROOT, (tWidget *)&g_sBackground);

    //
    // Set the default auto-repeat configuration of the push buttons.
    //
    ButtonsAutoRepeatSet(BUTTON_UP, 333, 100, 1, 100, 2, 100);
    ButtonsAutoRepeatSet(BUTTON_DOWN, 333, 100, 1, 100, 2, 100);
    ButtonsAutoRepeatSet(BUTTON_LEFT, 333, 100, 1, 100, 2, 100);
    ButtonsAutoRepeatSet(BUTTON_RIGHT, 333, 100, 1, 100, 2, 100);
    ButtonsAutoRepeatSet(BUTTON_SELECT, 0, 0, 0, 0, 0, 0);

    //
    // Display the splash screen.
    //
    DisplaySplash();

    //
    // The "Voltage Control Mode" panel should be displayed first.
    //
    ulPanel = PANEL_VOLTAGE;

    //
    // Loop forever.
    //
    while(1)
    {
        //
        // Determine which panel to display.
        //
        switch(ulPanel)
        {
            //
            // The "Voltage Control Mode" panel should be displayed.
            //
            case PANEL_VOLTAGE:
            {
                ulPanel = DisplayVoltage();
                break;
            }

            //
            // The "Device List" panel should be displayed.
            //
            case PANEL_DEV_LIST:
            {
                ulPanel = DisplayDevList();
                break;
            }

            //
            // The "Firmware Update" panel should be displayed.
            //
            case PANEL_UPDATE:
            {
                ulPanel = DisplayUpdate();
                break;
            }

            //
            // The "Help" panel should be displayed.
            //
            case PANEL_HELP:
            {
                ulPanel = DisplayHelp();
                break;
            }

            //
            // The "About" panel should be displayed.
            //
            case PANEL_ABOUT:
            {
                ulPanel = DisplayAbout();
                break;
            }
        }
    }
}
