//*****************************************************************************
//
// buttons.c - Routines for handling the on-board push buttons.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#include "../../../hw_memmap.h"
#include "../../../hw_types.h"
#include "../../../driverlib/gpio.h"
#include "../../../driverlib/sysctl.h"
#include "buttons.h"

//*****************************************************************************
//
// The debounced state of the five push buttons.
//
//*****************************************************************************
static unsigned char g_ucSwitches = 0x1f;

//*****************************************************************************
//
// The vertical counter used to debounce the push buttons.  The bit positions
// are the same as g_ucSwitches.
//
//*****************************************************************************
static unsigned char g_ucClockA = 0;
static unsigned char g_ucClockB = 0;
static unsigned char g_ucClockC = 0;

//*****************************************************************************
//
// This structure describes the auto-repeat configuration of a push button.
//
//*****************************************************************************
typedef struct
{
    unsigned char ucButton;
    unsigned char ucRepeatCount;
    unsigned short usCount;
    unsigned short usDelay;
    unsigned char ucRepeatCount1;
    unsigned char ucRepeatCount2;
    unsigned short usRepeatDelay1;
    unsigned short usRepeatDelay2;
    unsigned short usRepeatDelay3;
}
tButtons;

//*****************************************************************************
//
// The auto-repeat configuration for the push buttons.
//
//*****************************************************************************
static tButtons g_psButtons[] =
{
    { BUTTON_UP, 0, 0, 0, 0, 0, 0, 0, 0 },
    { BUTTON_DOWN, 0, 0, 0, 0, 0, 0, 0, 0 },
    { BUTTON_LEFT, 0, 0, 0, 0, 0, 0, 0, 0 },
    { BUTTON_RIGHT, 0, 0, 0, 0, 0, 0, 0, 0 },
    { BUTTON_SELECT, 0, 0, 0, 0, 0, 0, 0, 0 }
};

//*****************************************************************************
//
// The number of push buttons.
//
//*****************************************************************************
#define NUM_BUTTONS             (sizeof(g_psButtons) / sizeof(g_psButtons[0]))

//*****************************************************************************
//
// Initializes the push button driver.
//
//*****************************************************************************
void
ButtonsInit(void)
{
    //
    // Enable the peripherals used by the buttons.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);

    //
    // Configure the GPIOs used to read the state of the on-board push buttons.
    //
    GPIOPinTypeGPIOInput(GPIO_PORTF_BASE,
                         GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7);
    GPIOPadConfigSet(GPIO_PORTF_BASE,
                     GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7,
                     GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
    GPIOPinTypeGPIOInput(GPIO_PORTG_BASE, GPIO_PIN_4);
    GPIOPadConfigSet(GPIO_PORTG_BASE, GPIO_PIN_4, GPIO_STRENGTH_2MA,
                     GPIO_PIN_TYPE_STD_WPU);
}

//*****************************************************************************
//
// The button polling routine that must be called on a periodic basis.
//
//*****************************************************************************
unsigned char
ButtonsTick(void)
{
    unsigned long ulData, ulDelta;
    unsigned char ucReturn;

    //
    // Read the state of the push buttons.
    //
    ulData = ((GPIOPinRead(GPIO_PORTF_BASE, (GPIO_PIN_4 | GPIO_PIN_5 |
                                             GPIO_PIN_6 | GPIO_PIN_7)) >> 4) |
              GPIOPinRead(GPIO_PORTG_BASE, GPIO_PIN_4));

    //
    // Determine the switches that are at a different state than the debounced
    // state.
    //
    ulDelta = ulData ^ g_ucSwitches;

    //
    // Increment the clocks by one.
    //
    g_ucClockA ^= (g_ucClockB & g_ucClockC);
    g_ucClockB ^= g_ucClockC;
    g_ucClockC = ~g_ucClockC;

    //
    // Reset the clocks corresponding to switches that have not changed state.
    //
    g_ucClockA &= ulDelta;
    g_ucClockB &= ulDelta;
    g_ucClockC &= ulDelta;

    //
    // Get the new debounced switch state.
    //
    g_ucSwitches &= g_ucClockA | g_ucClockB | g_ucClockC;
    g_ucSwitches |= (~(g_ucClockA | g_ucClockB | g_ucClockC)) & ulData;

    //
    // Determine the switches that just changed debounced state.
    //
    ulDelta ^= (g_ucClockA | g_ucClockB | g_ucClockC);

    //
    // Loop through all of the buttons.
    //
    ucReturn = 0;
    for(ulData = 0; ulData < NUM_BUTTONS; ulData++)
    {
        //
        // Ignore this button if it is not pressed.
        //
        if(g_ucSwitches & (1 << ulData))
        {
            continue;
        }

        //
        // Return a button press for this button if it was just pressed.
        //
        if(ulDelta & (1 << ulData))
        {
            ucReturn |= 1 << ulData;
        }

        //
        // See if this button has auto-repeat enabled.
        //
        if(g_psButtons[ulData].usDelay != 0)
        {
            //
            // Reset the auto-repeat counters if this button was just pressed.
            //
            if(ulDelta & (1 << ulData))
            {
                g_psButtons[ulData].ucRepeatCount = 0;
                g_psButtons[ulData].usCount = g_psButtons[ulData].usDelay;
            }

            //
            // Decrement the auto-repeat count and see if it has reached zero.
            //
            if(--g_psButtons[ulData].usCount == 0)
            {
                //
                // Return a button press for this button.
                //
                ucReturn |= 1 << ulData;

                //
                // Increment the count of auto-repeats if it has not yet
                // reached 255.
                //
                if(g_psButtons[ulData].ucRepeatCount < 255)
                {
                    g_psButtons[ulData].ucRepeatCount++;
                }

                //
                // Reset the delay count based on the auto-repeat count.
                //
                if(g_psButtons[ulData].ucRepeatCount <=
                   g_psButtons[ulData].ucRepeatCount1)
                {
                    g_psButtons[ulData].usCount =
                        g_psButtons[ulData].usRepeatDelay1;
                }
                else if(g_psButtons[ulData].ucRepeatCount <=
                        g_psButtons[ulData].ucRepeatCount2)
                {
                    g_psButtons[ulData].usCount =
                        g_psButtons[ulData].usRepeatDelay2;
                }
                else
                {
                    g_psButtons[ulData].usCount =
                        g_psButtons[ulData].usRepeatDelay3;
                }
            }
        }
    }

    //
    // Return the set of buttons that were just pressed (or had phantom presses
    // as a result of auto-repeat).
    //
    return(ucReturn);
}

//*****************************************************************************
//
// Sets the auto-repeat configuration of a push button.
//
//*****************************************************************************
void
ButtonsAutoRepeatSet(unsigned char ucButton, unsigned short usDelay,
                     unsigned short usDelay1, unsigned char ucCount1,
                     unsigned short usDelay2, unsigned char ucCount2,
                     unsigned short usDelay3)
{
    unsigned long ulIdx;

    //
    // Loop through the push buttons.
    //
    for(ulIdx = 0; ulIdx < NUM_BUTTONS; ulIdx++)
    {
        //
        // See if this is the desired push button.
        //
        if(g_psButtons[ulIdx].ucButton == ucButton)
        {
            //
            // Update the configuration of this push button.
            //
            g_psButtons[ulIdx].usDelay = usDelay;
            g_psButtons[ulIdx].usRepeatDelay1 = usDelay1;
            g_psButtons[ulIdx].ucRepeatCount1 = ucCount1;
            g_psButtons[ulIdx].usRepeatDelay2 = usDelay2;
            g_psButtons[ulIdx].ucRepeatCount2 = ucCount2;
            g_psButtons[ulIdx].usRepeatDelay3 = usDelay3;
            return;
        }
    }
}
