//*****************************************************************************
//
// can_if.c - Functions that are used to interact with the CAN controller.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#include "../../../hw_types.h"
#include "../../../hw_memmap.h"
#include "../../../hw_ints.h"
#include "../../../hw_nvic.h"
#include "../../../driverlib/debug.h"
#include "../../../driverlib/interrupt.h"
#include "../../../driverlib/sysctl.h"
#include "../../../driverlib/gpio.h"
#include "../../../driverlib/can.h"
#include "../../../driverlib/pin_map.h"
#include "../../../driverlib/watchdog.h"
#include "../../../driverlib/rom.h"
#include "../can_proto.h"
#include "adc_ctrl.h"
#include "can_if.h"
#include "commands.h"
#include "constants.h"
#include "controller.h"
#include "hbridge.h"
#include "led.h"
#include "limit.h"
#include "param.h"

unsigned long g_ulSerialNumber;

//*****************************************************************************
//
// The firmware version.
//
//*****************************************************************************
//
// This variable holds the version code for the firmware.
//
const unsigned long g_ulFirmwareVersion = 3330;

//*****************************************************************************
//
// This structure holds any pending updates.
//
//*****************************************************************************
static struct
{
    //
    // The pending voltage update.
    //
    unsigned short usVoltage;

    //
    // The pending current update.
    //
    unsigned short usCurrent;

    //
    // The pending position update.
    //
    unsigned long ulPosition;

    //
    // The pending speed update.
    //
    unsigned long ulSpeed;

    //
    // The group number for a pending voltage update.
    //
    unsigned char ucVoltageGroup;

    //
    // The group number for a pending current update.
    //
    unsigned char ucCurrentGroup;

    //
    // The group number for a pending position update.
    //
    unsigned char ucPositionGroup;

    //
    // The group number for a pending speed update.
    //
    unsigned char ucSpeedGroup;
}
g_sPendingUpdates;

//*****************************************************************************
//
// The static enumeration data.
//
//*****************************************************************************
static const unsigned char g_pucEnumData[8] =
{
    //
    // Device Funciton, Motor Controller.
    //
    (unsigned char)(CAN_MSGID_DTYPE_MOTOR >> CAN_MSGID_DTYPE_S),

    //
    // Manufacturer, Luminary Micro.
    //
    (unsigned char)(CAN_MSGID_MFR_LM >> CAN_MSGID_MFR_S),

    //
    // Reserved values.
    //
    0x10, 0x20, 0x30, 0x40,

    //
    // Reserved values.
    //
    0, 0
};

//*****************************************************************************
//
// This structure holds the static defitions used to configure the CAN
// controller bit clock in the proper state given a requested setting.
// These are dustdevil based settings for 16MHz.
//
//*****************************************************************************
static const tCANBitClkParms CANBitClkSettings =
{
    5, 2, 2, 2
};

//*****************************************************************************
//
// These states are used to determine the mode during enumeration.
//
//*****************************************************************************
static enum
{
    //
    // This is the normal operating state.
    //
    STATE_IDLE,

    //
    // The controller has received a device assignement command and is waiting
    // to accept or reject the change.
    //
    STATE_ASSIGNMENT,

    //
    // The controller has received an enumeration request and is waiting to
    // send out the response.
    //
    STATE_ENUMERATE
}
eCANState;

//*****************************************************************************
//
// These globals hold the Event tick information to CAN related timing
// functions.
//
//*****************************************************************************
static unsigned long g_ulTickCount;
static unsigned long g_ulEventTick;

//*****************************************************************************
//
// The message object number and index to the local message object memory to
// use when accessing the messages.
//
//*****************************************************************************
#define MSG_OBJ_BCAST_RX        0
#define MSG_OBJ_BCAST_RX_ID     (MSG_OBJ_BCAST_RX + 1)
#define MSG_OBJ_BCAST_TX        1
#define MSG_OBJ_BCAST_TX_ID     (MSG_OBJ_BCAST_TX + 1)
#define MSG_OBJ_DEV_QUERY       2
#define MSG_OBJ_DEV_QUERY_ID    (MSG_OBJ_DEV_QUERY + 1)
#define MSG_OBJ_MC_RX           3
#define MSG_OBJ_MC_RX_ID        (MSG_OBJ_MC_RX + 1)
#define MSG_OBJ_FIRM_VER        4
#define MSG_OBJ_FIRM_VER_ID     (MSG_OBJ_FIRM_VER + 1)
#define MSG_OBJ_NUM_OBJECTS     5

//*****************************************************************************
//
// This holds the information for the data send message object that is used
// to send traffic on the CAN network.
//
//*****************************************************************************
static tCANMsgObject g_MsgObject[MSG_OBJ_NUM_OBJECTS];

//*****************************************************************************
//
// This is the buffer for data received by the broadcast message object.
//
//*****************************************************************************
static unsigned char g_pucBcastData[8];

//*****************************************************************************
//
// Global buffer used to receive buffer commands.
//
//*****************************************************************************
static unsigned long g_pulMCData[2] =
{
    0,
    0
};

//*****************************************************************************
//
// This holds the pending device number during the assignement state.
//
//*****************************************************************************
static unsigned char g_ucDevNumPending;

//*****************************************************************************
//
// This holds the pending device number during the assignement state.
//
//*****************************************************************************
static unsigned char g_ucFlags;

//
// This flag is set during startup and can be cleared only by another device
// sending a Power Status command.
//
#define CAN_FLAGS_POR           0x01

//*****************************************************************************
//
// Initial configuration of the network, this is just for the broadcast
// message configuration.
//
//*****************************************************************************
static void
CANConfigureNetwork(void)
{
    //
    // This message object will be used to send broadcast messages on the
    // CAN network.
    //
    g_MsgObject[MSG_OBJ_BCAST_TX].ulMsgID = 0;
    g_MsgObject[MSG_OBJ_BCAST_TX].ulMsgIDMask = 0;
    g_MsgObject[MSG_OBJ_BCAST_TX].ulFlags = MSG_OBJ_EXTENDED_ID;
    g_MsgObject[MSG_OBJ_BCAST_TX].ulMsgLen = 0;
    g_MsgObject[MSG_OBJ_BCAST_TX].pucMsgData = (unsigned char *)0xffffffff;

    //
    // This message object will be used to receive broadcast traffic on the
    // CAN network.
    //
    g_MsgObject[MSG_OBJ_BCAST_RX].ulMsgID = 0;

    //
    // Any API targeted at a devno of 0, manufacturer of 0, and device type
    // of 0.
    //
    g_MsgObject[MSG_OBJ_BCAST_RX].ulMsgIDMask =
        CAN_MSGID_DTYPE_M | CAN_MSGID_MFR_M | CAN_MSGID_DEVNO_M;

    //
    // Enable extended identifiers, interrupt and using extended message id
    // filtering.
    //
    g_MsgObject[MSG_OBJ_BCAST_RX].ulFlags =
        MSG_OBJ_RX_INT_ENABLE | MSG_OBJ_EXTENDED_ID | MSG_OBJ_USE_ID_FILTER |
        MSG_OBJ_USE_EXT_FILTER;

    //
    // No data will be read automatically, it will be read when it is received.
    //
    g_MsgObject[MSG_OBJ_BCAST_RX].ulMsgLen = 8;
    g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData = g_pucBcastData;

    //
    // Configure the broadcast receive message object.
    //
    CANMessageSet(CAN0_BASE, MSG_OBJ_BCAST_RX_ID,
                  &g_MsgObject[MSG_OBJ_BCAST_RX], MSG_OBJ_TYPE_RX);
}

//*****************************************************************************
//
// This function sends a message on the CAN bus.
//
// \param ulID is the CAN message identifier to transmit.
// \param pucData is a pointer to the data to transmit.
// \param ulSize is the number of bytes in the pucData buffer to transmit.
//
// This function is used to send response messages to the motor controller
// via the CAN bus.  This command will timeout if a previous message is failing
// to be sent out on the CAN bus.  Otherwise this function will schedule the
// CAN message to go out.
//
// \return This function will return zero to indicate successful scheduling
// of this message.  Any other value indicates that the message could not be
// sent at this time.
//
//*****************************************************************************
static unsigned long
CANSendBroadcastMsg(unsigned long ulId,  unsigned char *pucData,
                    unsigned long ulSize)
{
    int iIndex;
    unsigned long ulStatus;

    //
    // Set the number of times to retry reading the status before a timeout
    // and generating an error.
    //
    iIndex = 1000;

    //
    // Make sure the last transmission was successful.
    //
    while(1)
    {
        //
        // Wait for the transmit status to indicate that the previous message
        // was transmitted.
        //
        ulStatus = CANStatusGet(CAN0_BASE, CAN_STS_TXREQUEST);

        //
        // If the status ever goes to zero then exit the loop.
        //
        if(ulStatus == 0)
        {
            break;
        }

        //
        // If the timeout index ever goes to zero then exit the loop.
        //
        if(--iIndex == 0)
        {
            break;
        }
    }

    //
    // If there was no timeout then the interface is available so send message.
    //
    if(iIndex != 0)
    {
        //
        // Send the response packet.
        //
        g_MsgObject[MSG_OBJ_BCAST_TX].pucMsgData = pucData;
        g_MsgObject[MSG_OBJ_BCAST_TX].ulMsgLen = ulSize;
        g_MsgObject[MSG_OBJ_BCAST_TX].ulMsgID = ulId;
        CANMessageSet(CAN0_BASE, MSG_OBJ_BCAST_TX_ID,
                      &g_MsgObject[MSG_OBJ_BCAST_TX], MSG_OBJ_TYPE_TX);
    }
    else
    {
        //
        // The timeout reached zero so return with a non-zero value.
        //
        return(0xffffffff);
    }
    return(0);
}

//*****************************************************************************
//
// Configure the message objects based on a given device number.
//
// \param ucDevNum is the device number to use for receiving messages.
//
// This function is used to configure the CAN controller to receive messages
// for the device number passed in parameter ucDevNum.  If ucDevNum is zero
// then this function will reset all of the device number specific message
// object to not receive messages.  If ucDevNum is not zero then the message
// objects will be configured to receive messages based on the ucDevNum that
// is passed in.
//
// \return None.
//
//*****************************************************************************
static void
CANDeviceNumSet(unsigned char ucDevNum)
{
    //
    // Save the new device number.
    //
    ucDevNum = ucDevNum & CAN_MSGID_DEVNO_M;

    if(ucDevNum != 0)
    {
        //
        // This message object will be used to receive Motor control messages
        // on the CAN network.
        //
        g_MsgObject[MSG_OBJ_MC_RX].ulMsgID =
            CAN_MSGID_MFR_LM | CAN_MSGID_DTYPE_MOTOR | ucDevNum;

        //
        // Set the filter mask to look at the device identifier, manufacturer,
        // and the device type.
        //
        g_MsgObject[MSG_OBJ_MC_RX].ulMsgIDMask =
            CAN_MSGID_DEVNO_M | CAN_MSGID_MFR_M | CAN_MSGID_DTYPE_M;

        //
        // Enable filtering based on the mask.
        //
        g_MsgObject[MSG_OBJ_MC_RX].ulFlags =
            MSG_OBJ_RX_INT_ENABLE | MSG_OBJ_EXTENDED_ID |
            MSG_OBJ_USE_ID_FILTER | MSG_OBJ_USE_EXT_FILTER;

        //
        // This just initializes the data pointer and does nothing else.
        //
        g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen = 8;
        g_MsgObject[MSG_OBJ_MC_RX].pucMsgData = (unsigned char *)g_pulMCData;

        //
        // Configure the motor control message receive message object.
        //
        CANMessageSet(CAN0_BASE, MSG_OBJ_MC_RX_ID, &g_MsgObject[MSG_OBJ_MC_RX],
                      MSG_OBJ_TYPE_RX);

        //
        // Configure the auto-responding device indentification data message
        // object.
        //
        g_MsgObject[MSG_OBJ_DEV_QUERY].ulMsgID = CAN_MSGID_API_DEVQUERY |
                                                 ucDevNum;

        //
        // Only look at the API and Device identifier.
        //
        g_MsgObject[MSG_OBJ_DEV_QUERY].ulMsgIDMask = CAN_MSGID_FULL_M;

        //
        // Enable filtering based on the mask.
        //
        g_MsgObject[MSG_OBJ_DEV_QUERY].ulFlags = MSG_OBJ_EXTENDED_ID |
                                                 MSG_OBJ_USE_ID_FILTER |
                                                 MSG_OBJ_USE_EXT_FILTER;

        //
        // The identification data has 8 bytes of data so initialize the values.
        //
        g_MsgObject[MSG_OBJ_DEV_QUERY].ulMsgLen = 8;
        g_MsgObject[MSG_OBJ_DEV_QUERY].pucMsgData =
            (unsigned char *)g_pucEnumData;

        //
        // Send the configuration to the device identification message object.
        //
        CANMessageSet(CAN0_BASE, MSG_OBJ_DEV_QUERY_ID,
                      &g_MsgObject[MSG_OBJ_DEV_QUERY],
                      MSG_OBJ_TYPE_RXTX_REMOTE);

        //
        // Configure the auto-responding firmware version message object to
        // look for a fully specified message identifier.
        //
        g_MsgObject[MSG_OBJ_FIRM_VER].ulMsgID =
            CAN_MSGID_API_FIRMVER | ucDevNum;
        g_MsgObject[MSG_OBJ_FIRM_VER].ulMsgIDMask = CAN_MSGID_FULL_M;

        //
        // Enable filtering based on the mask.
        //
        g_MsgObject[MSG_OBJ_FIRM_VER].ulFlags = MSG_OBJ_EXTENDED_ID |
                                                MSG_OBJ_USE_ID_FILTER |
                                                MSG_OBJ_USE_EXT_FILTER;

        //
        // Set the firmware version response to 4 bytes long and initialize
        // the data to the value in g_ulFirmwareVerion.
        //
        g_MsgObject[MSG_OBJ_FIRM_VER].ulMsgLen = 4;
        g_MsgObject[MSG_OBJ_FIRM_VER].pucMsgData =
            (unsigned char *)&g_ulFirmwareVersion;

        //
        // Send the configuration to the Firmware message object.
        //
        CANMessageSet(CAN0_BASE, MSG_OBJ_FIRM_VER_ID,
                      &g_MsgObject[MSG_OBJ_FIRM_VER],
                      MSG_OBJ_TYPE_RXTX_REMOTE);

        //
        // If previously in the assignement state then return to the IDLE
        // state since the next command will take the bus out of the
        // assignement state.
        //
        if(eCANState == STATE_ASSIGNMENT)
        {
            eCANState = STATE_IDLE;
        }
    }
    else
    {
        //
        // Reset the state of all the message object and the state of the CAN
        // module to a known state.
        //
        CANInit(CAN0_BASE);

        //
        // Set up the message object(s) that will receive messages on the CAN
        // bus.
        //
        CANConfigureNetwork();

        //
        // Take the CAN1 device out of INIT state.
        //
        CANEnable(CAN0_BASE);

        //
        // Enable interrups from CAN controller.
        //
        CANIntEnable(CAN0_BASE, CAN_INT_MASTER | CAN_INT_ERROR);
    }

    //
    // If the value has changed then it needs to be saved.
    //
    if(ucDevNum != g_sParameters.ucDeviceNumber)
    {
        //
        // Save the pending change.
        //
        g_sParameters.ucDeviceNumber = ucDevNum;

        //
        // Save the new a new ID.
        //
        ParamSave();
    }
}

//*****************************************************************************
//
// This function handles CAN Status API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN status API message is received.  If
// necessary, it will retrieve the information requested from the board and
// return it to the requestor.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
StatusHandler(unsigned long ulID, unsigned char *pucData,
              unsigned long ulMsgLen)
{
    unsigned long ulData;
    unsigned char ucData;

    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        case LM_API_STATUS_VOLTOUT:
        {
            //
            // Send back the motor output voltage.
            //
            ulData = ControllerVoltageGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 2);
            break;
        }

        case LM_API_STATUS_VOLTBUS:
        {
            //
            // Send back the current bus voltage.
            //
            ulData = ADCVBusGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 2);
            break;
        }

        case LM_API_STATUS_FAULT:
        {
            //
            // Send back the active faults.
            //
            ulData = ControllerFaultsActive();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 2);
            break;
        }

        case LM_API_STATUS_CURRENT:
        {
            //
            // Send back the motor current.
            //
            ulData = ADCCurrentGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 2);
            break;
        }

        case LM_API_STATUS_TEMP:
        {
            //
            // Send back the temperature.
            //
            ulData = ADCTemperatureGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 2);
            break;
        }

        case LM_API_STATUS_POS:
        {
            //
            // Send back the motor position.
            //
            ulData = ControllerPositionGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 4);
            break;
        }

        case LM_API_STATUS_SPD:
        {
            //
            // Send back the motor speed.
            //
            ulData = ControllerSpeedGet();
            CANSendBroadcastMsg(ulID, (unsigned char *)&ulData, 4);
            break;
        }

        case LM_API_STATUS_LIMIT:
        {
            //
            // Default is that the limits are not "good".
            //
            ucData = 0;

            //
            // See if the forward limit is in a "good" state.
            //
            if(LimitForwardOK())
            {
                ucData |= LM_STATUS_LIMIT_FWD;
            }

            //
            // See if the reverse limit is in a "good" state.
            //
            if(LimitReverseOK())
            {
                ucData |= LM_STATUS_LIMIT_REV;
            }

            //
            // Send back the limit switch state.
            //
            CANSendBroadcastMsg(ulID, &ucData, 1);
            break;
        }

        case LM_API_STATUS_POWER:
        {
            if(ulMsgLen == 0)
            {
                //
                // Send back the power state flags.
                //
                CANSendBroadcastMsg(ulID, &g_ucFlags, 1);
            }
            else if(ulMsgLen == 1)
            {
                //
                // Only POR can be cleared at this time.
                //
                if(pucData[0] & CAN_FLAGS_POR)
                {
                    g_ucFlags &= ~CAN_FLAGS_POR;
                }
            }
            break;
        }

        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function handles CAN Voltage API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN voltage API message is received.  This
// function will parse the voltage command and call the appropriate motor
// controller function.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
VoltageHandler(unsigned long ulID, unsigned char * pucData,
               unsigned long ulMsgLen)
{
    unsigned short *pusData;
    short *psData;

    //
    // Create a local pointer to avoid later type casting.
    //
    pusData = (unsigned short *)pucData;
    psData = (short *)pucData;

    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        //
        // Enable Voltage control mode.
        //
        case LM_API_VOLT_EN:
        {
            //
            // Set Voltage Mode.
            //
            CommandVoltageMode(true);

            //
            // Reset pending updates when switching modes.
            //
            g_sPendingUpdates.ucCurrentGroup = 0;
            g_sPendingUpdates.ucVoltageGroup = 0;
            g_sPendingUpdates.ucPositionGroup = 0;
            g_sPendingUpdates.ucSpeedGroup = 0;

            break;
        }

        //
        // Disable voltage control mode.
        //
        case LM_API_VOLT_DIS:
        {
            CommandVoltageMode(false);

            break;
        }

        //
        // Motor controller Set Output Voltage.
        //
        case LM_API_VOLT_SET:
        {
            //
            // If there was either no group specified or if the value specified
            // was zero then update the voltage, otherwise the voltage update
            // is pending until it is committed.
            //
            if((ulMsgLen <= 2) || (pucData[2] == 0))
            {
                //
                // Send the volatage on to the handler.
                //
                CommandVoltageSet(psData[0]);
            }
            else
            {
                //
                // Save the voltage setting and the group.
                //
                g_sPendingUpdates.usVoltage = psData[0];
                g_sPendingUpdates.ucVoltageGroup = pucData[2];
            }
            break;
        }

        //
        // Motor controller Set Voltage Ramp Rate received.
        //
        case LM_API_VOLT_SET_RAMP:
        {
            CommandVoltageRateSet(pusData[0]);

            break;
        }

        //
        // Unknown command.
        //
        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function handles CAN Speed API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN speed API message is received.  This
// function will parse the speed command and call the appropriate motor
// controller function.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
SpeedHandler(unsigned long ulID, unsigned char *pucData,
             unsigned long ulMsgLen)
{
    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        //
        // Enable speed control mode.
        //
        case LM_API_SPD_EN:
        {
            //
            // Enable speed mode.
            //
            CommandSpeedMode(true);

            //
            // Reset pending updates when switching modes.
            //
            g_sPendingUpdates.ucCurrentGroup = 0;
            g_sPendingUpdates.ucVoltageGroup = 0;
            g_sPendingUpdates.ucPositionGroup = 0;
            g_sPendingUpdates.ucSpeedGroup = 0;

            break;
        }

        //
        // Disable speed control mode.
        //
        case LM_API_SPD_DIS:
        {
            //
            // Disable speed mode.
            //
            CommandSpeedMode(false);

            break;
        }

        //
        // Set the Speed.
        //
        case LM_API_SPD_SET:
        {
            //
            // If there was either no group specified or if the value specified
            // was zero then update the speed, otherwise the speed update
            // is pending until it is committed.
            //
            if((ulMsgLen <= 4) || (pucData[4] == 0))
            {
                //
                // When read as an unsigned short the value in pucData is the
                // rotational speed in revolution per second.
                //
                CommandSpeedSet(*(unsigned long *)pucData);
            }
            else
            {
                //
                // Save the speed setting and the group.
                //
                g_sPendingUpdates.ulSpeed = *(unsigned long *)pucData;
                g_sPendingUpdates.ucSpeedGroup = pucData[4];
            }

            break;
        }

        //
        // Set the proportional constant used in the PID algorithm.
        //
        case LM_API_SPD_PC:
        {
            //
            // The proportional constant is 32 bits.
            //
            CommandSpeedPSet(*(unsigned long *)pucData);

            break;
        }

        //
        // Set the integral constant used in the PID algorithm.
        //
        case LM_API_SPD_IC:
        {
            //
            // The integral constant is 32 bits.
            //
            CommandSpeedISet(*(unsigned long *)pucData);

            break;
        }

        //
        // Set the differential constant used in the PID algorithm.
        //
        case LM_API_SPD_DC:
        {
            //
            // The differential constant is 32 bits.
            //
            CommandSpeedDSet(*(unsigned long *)pucData);

            break;
        }

        //
        // Set the speed measurement reference.
        //
        case LM_API_SPD_REF:
        {
            //
            // The speed reference is 8 bits.
            //
            CommandSpeedSrcSet(*pucData);

            break;
        }

        //
        // Unknown command.
        //
        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function handles CAN position API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN position API message is received.  This
// function will parse the position command and call the appropriate motor
// controller function.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
PositionHandler(unsigned long ulID, unsigned char *pucData,
                unsigned long ulMsgLen)
{
    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        //
        // Enable position control mode.
        //
        case LM_API_POS_EN:
        {
            //
            // The 32 bit value in pucData holds the initial position.
            // The motor controller should set its output voltage to
            // 0V(neutral)
            //
            CommandPositionMode(true, *(long *)pucData);

            //
            // Reset pending updates when switching modes.
            //
            g_sPendingUpdates.ucCurrentGroup = 0;
            g_sPendingUpdates.ucVoltageGroup = 0;
            g_sPendingUpdates.ucPositionGroup = 0;
            g_sPendingUpdates.ucSpeedGroup = 0;

            break;
        }

        //
        // Disable position control mode.
        //
        case LM_API_POS_DIS:
        {
            //
            // Switch out of position mode and back to the default mode.
            //
            CommandPositionMode(false, 0);

            break;
        }

        //
        // Set the target shaft position.
        //
        case LM_API_POS_SET:
        {
            //
            // If there was either no group specified or if the value specified
            // was zero then update the position, otherwise the position update
            // is pending until it is committed.
            //
            if((ulMsgLen <= 4) || (pucData[4] == 0))
            {
                //
                // Send the 32 bit position to move to.
                //
                CommandPositionSet(*(long *)pucData);
            }
            else
            {
                //
                // Save the position setting and the group.
                //
                g_sPendingUpdates.ulPosition = *(unsigned long *)pucData;
                g_sPendingUpdates.ucPositionGroup = pucData[4];
            }

            break;
        }

        //
        // Set the proportional constant used in the PID algorithm.
        //
        case LM_API_POS_PC:
        {
            //
            // The proportional constant is 32 bits.
            //
            CommandPositionPSet(*(long *)pucData);

            break;
        }

        //
        // Set the integral constant used in the PID algorithm.
        //
        case LM_API_POS_IC:
        {
            //
            // The integral constant is 32 bits.
            //
            CommandPositionISet(*(long *)pucData);

            break;
        }

        //
        // Set the differential constant used in the PID algorithm.
        //
        case LM_API_POS_DC:
        {
            //
            // The differential constant is 32 bits.
            //
            CommandPositionDSet(*(long *)pucData);

            break;
        }

        //
        // Set the the reference measurement source for position measurement.
        //
        case LM_API_POS_REF:
        {
            //
            // The position reference is 8 bits.
            //
            CommandPositionSrcSet(*pucData);

            break;
        }

        //
        // Unknown command.
        //
        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function handles CAN current API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN current API message is received.  This
// function will parse the current command and call the appropriate motor
// controller function.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
CurrentHandler(unsigned long ulID, unsigned char *pucData,
                  unsigned long ulMsgLen)
{
    unsigned long *pulData;
    short *psData;

    //
    // Create some local pointers to avoid later type casting.
    //
    pulData = (unsigned long *)pucData;
    psData = (short *)pucData;

    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        //
        // Enable the current control mode.
        //
        case LM_API_ICTRL_EN:
        {
            //
            // Enable current control mode.
            //
            CommandCurrentMode(true);

            //
            // Reset pending updates when switching modes.
            //
            g_sPendingUpdates.ucCurrentGroup = 0;
            g_sPendingUpdates.ucVoltageGroup = 0;
            g_sPendingUpdates.ucPositionGroup = 0;
            g_sPendingUpdates.ucSpeedGroup = 0;

            break;
        }

        //
        // Disable the current control mode.
        //
        case LM_API_ICTRL_DIS:
        {
            //
            // Disable current control mode and return to the default mode.
            //
            CommandCurrentMode(false);

            break;
        }

        //
        // Set the target winding current for the motor.
        //
        case LM_API_ICTRL_SET:
        {
            //
            // If there was either no group specified or if the value specified
            // was zero then update the current, otherwise the current update
            // is pending until it is committed.
            //
            if((ulMsgLen <= 2) || (pucData[2] == 0))
            {
                //
                // The value in pucData is a 10 bit number that specifies the
                // set point value for the current multiplied by 60mA.
                //
                CommandCurrentSet(psData[0]);
            }
            else
            {
                //
                // Save the current setting and the group.
                //
                g_sPendingUpdates.usCurrent = psData[0];
                g_sPendingUpdates.ucCurrentGroup = pucData[2];
            }
            break;
        }

        //
        // Set the proportional constant used in the PID algorithm.
        //
        case LM_API_ICTRL_PC:
        {
            //
            // The proportional constant is 32 bits.
            //
            CommandCurrentPSet(pulData[0]);
            break;
        }

        //
        // Set the integral constant used in the PID algorithm.
        //
        case LM_API_ICTRL_IC:
        {
            //
            // The integral constant is 32 bits.
            //
            CommandCurrentISet(pulData[0]);

            break;
        }

        //
        // Set the differential constant used in the PID algorithm.
        //
        case LM_API_ICTRL_DC:
        {
            //
            // The differential constant is 32 bits.
            //
            CommandCurrentDSet(pulData[0]);

            break;
        }

        //
        // Unknown command.
        //
        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function handles CAN configuration API messages.
//
// \param ulID is the fully specified ID that was received.
// \param pucData is a buffer that holds the data that was received.
// \param ulMsgLen is the number of valid bytes of data in pucData buffer.
//
// This function is called when a CAN configuration API message is received.
// This function will parse the configuration command and call the appropriate
// motor controller function.
//
// \return The function will return zero if the command was understood and it
// will return non-zero for unsupported commands.
//
//*****************************************************************************
static unsigned long
ConfigHandler(unsigned long ulID, unsigned char * pucData,
              unsigned long ulMsgLen)
{
    //
    // Mask out the device number and see what the command is.
    //
    switch(ulID & (~CAN_MSGID_DEVNO_M))
    {
        //
        // This was the Luminary Micro API for setting the number of brushes.
        //
        case LM_API_CFG_NUM_BRUSHES:
        {
            //
            // Set the number of brushes.
            //
            CommandNumBrushesSet(*pucData);

            break;
        }

        //
        // This was the Luminary Micro API for setting the number of encoder
        // lines.
        //
        case LM_API_CFG_ENC_LINES:
        {
            //
            // Set the number of encoder lines.
            //
            CommandEncoderLinesSet(*(unsigned short *)pucData);

            break;
        }

        //
        // This was the Luminary Micro API for setting the number of
        // turns in the potentiometer.
        //
        case LM_API_CFG_POT_TURNS:
        {
            //
            // Set the number of potentiometer turns.
            //
            CommandPotTurnsSet(*(unsigned short *)pucData);

            break;
        }

        //
        // This was the Luminary Micro API for setting the breaking mode to
        // coast or break.
        //
        case LM_API_CFG_BRAKE_COAST:
        {
            //
            // Set the brake/coast mode.
            //
            CommandBrakeCoastSet(*pucData);

            break;
        }

        //
        // This was the Luminary Micro API for enabling the limit switches.
        //
        case LM_API_CFG_LIMIT_MODE:
        {
            //
            // Configure the position limit switches.
            //
            CommandPositionLimitMode(*pucData);

            break;
        }

        //
        // This was the Luminary Micro API for setting the forward position
        // limit "switch".
        //
        case LM_API_CFG_LIMIT_FWD:
        {
            //
            // Set the forward position limit switch.
            //
            CommandPositionLimitForwardSet(*(unsigned long *)pucData,
                                           pucData[4]);

            break;
        }

        //
        // This was the Luminary Micro API for setting the reverse position
        // limit switch.
        //
        case LM_API_CFG_LIMIT_REV:
        {
            //
            // Set the reverse position limit switch.
            //
            CommandPositionLimitReverseSet(*(unsigned long *)pucData,
                                           pucData[4]);

            break;
        }

        default:
        {
            return(0xffffffff);
        }
    }
    return(0);
}

//*****************************************************************************
//
// This function configures the CAN hardware and the basic message objects so
// that the interface is ready to use once the application returns from this
// function.
//
// \return None.
//
//*****************************************************************************
void
CANIFInit(void)
{
    //
    // Default state is idle.
    //
    eCANState = STATE_IDLE;

    //
    // Configure CAN Pins
    //
    ROM_SysCtlPeripheralEnable(CAN0RX_PERIPH);
    ROM_GPIOPinTypeCAN(CAN0RX_PORT, CAN0RX_PIN | CAN0TX_PIN);

    //
    // Enable the CAN controllers.
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_CAN0);

    //
    // Make sure all pending updates are canceled.
    //
    g_sPendingUpdates.ucCurrentGroup = 0;
    g_sPendingUpdates.ucVoltageGroup = 0;
    g_sPendingUpdates.ucPositionGroup = 0;
    g_sPendingUpdates.ucSpeedGroup = 0;

    //
    // Reset the global flags.
    //
    g_ucFlags = CAN_FLAGS_POR;

    //
    // Reset the state of all the message object and the state of the CAN
    // module to a known state.
    //
    CANInit(CAN0_BASE);

    //
    // Configure the bit clock parameters for the CAN device.
    //
    CANBitTimingSet(CAN0_BASE, (tCANBitClkParms *)&CANBitClkSettings);

    //
    // Take the CAN0 device out of INIT state.
    //
    CANEnable(CAN0_BASE);

    //
    // Enable interrups from CAN controller.
    //
    CANIntEnable(CAN0_BASE, CAN_INT_MASTER | CAN_INT_ERROR);

    //
    // Set up the message object(s) that will receive messages on the CAN bus.
    //
    CANConfigureNetwork();

    //
    // Enable auto-retry on CAN transmit.
    //
    CANRetrySet(CAN0_BASE, true);

    //
    // If the initial device number is not zero then configure the CAN to use
    // the saved device id.
    //
    if(g_sParameters.ucDeviceNumber != 0)
    {
        //
        // Configure the message objects that are based on the device number.
        //
        CANDeviceNumSet(g_sParameters.ucDeviceNumber);
    }

    //
    // Enable the CAN0 interrupts.
    //
    ROM_IntEnable(INT_CAN0);
}

//*****************************************************************************
//
// This function is called by external functions when a button press has been
// detected.  If the communication link is via CAN bus and the interface was in
// the Assignment state, then the CAN controller will accept the current
// device identifier assignement and store the value in the parameter block.
//
// \return None.
//
//*****************************************************************************
void
CANIFButtonPress(void)
{
    //
    // Ignore the button press if the system is using the servo connection.
    //
    if(ControllerLinkType() == LINK_TYPE_SERVO)
    {
        return;
    }

    //
    // If the device was in the assignment state then save the new value.
    //
    if(eCANState == STATE_ASSIGNMENT)
    {
        //
        // Set the configuration of the message objects.
        //
        CANDeviceNumSet(g_ucDevNumPending);

        //
        // Leave the assignement state.
        //
        eCANState = STATE_IDLE;
    }

    //
    // Blink the device ID.  This acknowledges an ID assignment, and if not in
    // assignment mode provides a way to determine the ID.
    //
    LEDBlinkID();
}

//*****************************************************************************
//
// The CAN interface interrupt handler.
//
// This function handles all interrupts from the can controller and parses out
// the various commands to their appropriate handlers.
//
// \return None.
//
//*****************************************************************************
void
CAN0IntHandler(void)
{
    unsigned long ulStatus, ulAck, ulGroup;

    //
    // Find the cause of the interrupt, if it is a status interrupt then just
    // acknowledge the interrupt by reading the status register.
    //
    ulStatus = CANIntStatus(CAN0_BASE, CAN_INT_STS_CAUSE);

    //
    // Clear the interrupt.
    //
    CANIntClear(CAN0_BASE, ulStatus);

    //
    // This resets the watchdog timeout for CAN messages.
    //
    ControllerLinkGood(LINK_TYPE_CAN);

    //
    // The ulStatus variable will be the message identifier that cause the
    // interrupt or it will indicate a status interrupt.
    //
    switch(ulStatus)
    {
        //
        // This message object is used to receive broadcast messages.
        //
        case MSG_OBJ_BCAST_RX_ID:
        {
            //
            // Retrieve the CAN message and process it.
            //
            CANMessageGet(CAN0_BASE, MSG_OBJ_BCAST_RX_ID,
                          &g_MsgObject[MSG_OBJ_BCAST_RX],
                          true);

            //
            // The API requested is determined based on the message identifier
            // that was recevied.
            //
            switch(g_MsgObject[MSG_OBJ_BCAST_RX].ulMsgID)
            {
                //
                // A system halt request was received.
                //
                case CAN_MSGID_API_SYSHALT:
                {
                    //
                    // Reset pending updates when switching modes.
                    //
                    g_sPendingUpdates.ucCurrentGroup = 0;
                    g_sPendingUpdates.ucVoltageGroup = 0;
                    g_sPendingUpdates.ucPositionGroup = 0;
                    g_sPendingUpdates.ucSpeedGroup = 0;

                    //
                    // Got a System Halt! Set motor to neutral.
                    //
                    CommandVoltageSet(0);
                    CommandCurrentSet(0);
                    CANSendBroadcastMsg(LM_API_STATUS_VOLTOUT, 0, 0);

                    break;
                }

                //
                // A system reset request was received.
                //
                case CAN_MSGID_API_SYSRST:
                {
                    //
                    // Reset pending updates when switching modes.
                    //
                    g_sPendingUpdates.ucCurrentGroup = 0;
                    g_sPendingUpdates.ucVoltageGroup = 0;
                    g_sPendingUpdates.ucPositionGroup = 0;
                    g_sPendingUpdates.ucSpeedGroup = 0;

                    //
                    // Got a System Reset! Set motor to neutral.
                    //
                    CommandVoltageSet(0);
                    CommandCurrentSet(0);

                    break;
                }

                //
                // A enumeration request was received.
                //
                case CAN_MSGID_API_ENUMERATE:
                {
                    //
                    // If there is no device number currently set, then don't
                    // wait to send the enumeration response as this controller
                    // has no device identifier.
                    //
                    if(g_sParameters.ucDeviceNumber != 0)
                    {
                        //
                        // Switch to the enumeration state to wait to send
                        // out the enumeration data.
                        //
                        eCANState = STATE_ENUMERATE;

                        //
                        // Wait 1ms * the current device number.
                        //
                        g_ulEventTick = g_ulTickCount +
                            ((UPDATES_PER_SECOND *
                              g_sParameters.ucDeviceNumber) / 1000);
                    }
                    break;
                }

                //
                // This was a request to assign a new device identifier.
                //
                case CAN_MSGID_API_DEVASSIGN:
                {
                    //
                    // If an out of bounds device ID was specified then ignore
                    // the request.
                    //
                    if(g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData[0] >
                       CAN_MSGID_DEVNO_M)
                    {
                    }
                    else if(g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData[0] != 0)
                    {
                        //
                        // Save the pending address.
                        //
                        g_ucDevNumPending =
                            g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData[0];

                        //
                        // This is pending until commited.
                        //
                        eCANState = STATE_ASSIGNMENT;

                        //
                        // Set the tick that will trigger leaving assignment
                        // mode.
                        //
                        g_ulEventTick = g_ulTickCount +
                            (UPDATES_PER_SECOND * CAN_ASSIGN_WAIT_SECONDS);

                        //
                        // Let the world know that Assigment state has started.
                        //
                        LEDAssignStart();
                    }
                    else
                    {
                        //
                        // Set the configuration of the message objects and
                        // the device identifier to zero immediately.
                        //
                        CANDeviceNumSet(0);
                    }

                    break;
                }
                case CAN_MSGID_API_UPDATE:
                {
                    //
                    // Check if there is an ID to update and if it belongs to
                    // this board.
                    //
                    if((g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData[0] !=
                        g_sParameters.ucDeviceNumber) ||
                       (g_MsgObject[MSG_OBJ_BCAST_RX].ulMsgLen != 1))
                    {
                        break;
                    }

                    //
                    // Disable all processor interrupts.  Instead of disabling
                    // them one at a time (and possibly missing an interrupt
                    // if new sources are added), a direct write to NVIC is
                    // done to disable all peripheral interrupts.
                    //
                    HWREG(NVIC_DIS0) = 0xffffffff;
                    HWREG(NVIC_DIS1) = 0xffffffff;

                    //
                    // Clear any active interrupts.  If the boot loader uses
                    // any interrupts, they will therefore respond as expected.
                    //
                    HWREG(NVIC_APINT) = (NVIC_APINT_VECTKEY |
                                         NVIC_APINT_VECT_CLR_ACT);

                    //
                    // Set the HBridge to a safe state before the firmware
                    // update starts.
                    //
                    HBridgeFirmwareUpdate();

                    //
                    // Set the LED to indicate that a firmware update is
                    // occurring.
                    //
                    LEDFirmwareUpdate();

                    //
                    // Call the boot loader.
                    //
                    (*((void (*)(void))(*(unsigned long *)0x2c)))();

                    //
                    // Loop forever.  This should never be reached.
                    //
                    while(1)
                    {
                    }
                }

                //
                // Handle the sync commands.
                //
                case CAN_MSGID_API_SYNC:
                {
                    //
                    // Retrieve the group identifier.
                    //
                    ulGroup = g_MsgObject[MSG_OBJ_BCAST_RX].pucMsgData[0];

                    //
                    // If there are pending voltage updates then set the
                    // values now.
                    //
                    if((g_sPendingUpdates.ucVoltageGroup & ulGroup) != 0)
                    {
                        //
                        // Send the volatage on to the handler.
                        //
                        CommandVoltageSet(g_sPendingUpdates.usVoltage);

                        //
                        // Update is no longer pending.
                        //
                        g_sPendingUpdates.ucVoltageGroup = 0;
                    }

                    //
                    // If there are pending current updates then set the
                    // values now.
                    //
                    if((g_sPendingUpdates.ucCurrentGroup & ulGroup) != 0)
                    {
                        //
                        // Send the current on to the handler.
                        //
                        CommandCurrentSet(g_sPendingUpdates.usCurrent);

                        //
                        // Update is no longer pending.
                        //
                        g_sPendingUpdates.ucCurrentGroup = 0;
                    }

                    //
                    // If there are pending speed updates then set the
                    // values now.
                    //
                    if((g_sPendingUpdates.ucSpeedGroup & ulGroup) != 0)
                    {
                        //
                        // Send the speed setting to the handler.
                        //
                        CommandSpeedSet(g_sPendingUpdates.ulSpeed);

                        //
                        // Update is no longer pending.
                        //
                        g_sPendingUpdates.ucSpeedGroup = 0;
                    }

                    //
                    // If there are pending position updates then set the
                    // values now.
                    //
                    if((g_sPendingUpdates.ucPositionGroup & ulGroup) != 0)
                    {
                        //
                        // Send the speed setting to the handler.
                        //
                        CommandPositionSet(g_sPendingUpdates.ulPosition);

                        //
                        // Update is no longer pending.
                        //
                        g_sPendingUpdates.ucPositionGroup = 0;
                    }

                    break;
                }

                //
                // Nothing is done in response to a heartbeat command, this
                // just causes the controller to hit the watchdog below.
                //
                case CAN_MSGID_API_HEARTBEAT:
                {
                    break;
                }

                default:
                {
                    break;
                }
            }
            break;
        }

        //
        // The broagcast Transmit interrupt was received.  This is a stub as
        // this intterrupt should not occur.
        //
        case MSG_OBJ_BCAST_TX_ID:
        {
            break;
        }

        //
        // This message object is used to receive motor control commands.
        //
        case MSG_OBJ_MC_RX_ID:
        {
            //
            // By default ack the command.
            //
            ulAck = 1;

            //
            // Retrieve the CAN message and process it.
            //
            CANMessageGet(CAN0_BASE, MSG_OBJ_MC_RX_ID,
                          &g_MsgObject[MSG_OBJ_MC_RX], true);

            switch(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID & CAN_MSGID_API_CLASS_M)
            {
                //
                // Voltage motor control commands.
                //
                case CAN_API_MC_VOLTAGE:
                {
                    //
                    // Call the Voltage handler.
                    //
                    VoltageHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                   g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                   g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Handle the Speed commands.
                //
                case CAN_API_MC_SPD:
                {
                    //
                    // Call the speed handler.
                    //
                    SpeedHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                 g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                 g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Handle the position control class commands.
                //
                case CAN_API_MC_POS:
                {
                    //
                    // Call the position command handler.
                    //
                    PositionHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                    g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                    g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Handle the Current control class commands.
                //
                case CAN_API_MC_ICTRL:
                {
                    //
                    // Call the current command handler.
                    //
                    CurrentHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                   g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                   g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Handle the Get status command.
                //
                case CAN_API_MC_STATUS:
                {
                    //
                    // Call the status command handler.
                    //
                    StatusHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                  g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                  g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Handle the Get status command.
                //
                case CAN_API_MC_CFG:
                {
                    //
                    // Call the status command handler.
                    //
                    ConfigHandler(g_MsgObject[MSG_OBJ_MC_RX].ulMsgID,
                                  g_MsgObject[MSG_OBJ_MC_RX].pucMsgData,
                                  g_MsgObject[MSG_OBJ_MC_RX].ulMsgLen);
                    break;
                }

                //
                // Unknown command class.
                //
                default:
                {
                    ulAck = 0;
                    break;
                }
            }

            //
            // Only send out an ack for a recognized command.
            //
            if(ulAck != 0)
            {
                //
                // Send out the ACK.
                //
                CANSendBroadcastMsg(
                    LM_API_ACK |
                    (g_MsgObject[MSG_OBJ_MC_RX].ulMsgID & CAN_MSGID_DEVNO_M),
                    0, 0);
            }

            break;
        }
        default:
        {
            break;
        }
    }

    //
    // Delay the watchdog interrupt since a CAN command was received.
    //
    ROM_WatchdogIntClear(WATCHDOG_BASE);
}

//*****************************************************************************
//
// This performs any timing releated operations needed by the CAN interface.
//
// When the CAN controller is in STATE_ASSIGNMENT, this allows the controller
// to time out after CAN_ASSIGN_WAIT_SECONDS seconds and resume normal
// operation.  When the CAN controller is in STATE_ENUMERATE state this allows
// the controller to wait the specified time before responding to the
// enumeration request to prevent all controllers on a network from responding
// at once.
//
//*****************************************************************************
void
CANIFTick(void)
{
    //
    // Ignore the tick if the system is using the servo connection.
    //
    if(ControllerLinkType() == LINK_TYPE_SERVO)
    {
        return;
    }

    //
    // Increment the tick count.
    //
    g_ulTickCount++;

    //
    // Assignment state is handled in this case.
    //
    if(eCANState == STATE_ASSIGNMENT)
    {
        //
        // Wait for the correct tick time.
        //
        if(g_ulEventTick == g_ulTickCount)
        {
            //
            // Return to the idle state.
            //
            eCANState = STATE_IDLE;

            //
            // If the pending change was not accepted and was the the same as
            // it was before then set the device number to 0 and accept it.
            //
            if(g_ucDevNumPending == g_sParameters.ucDeviceNumber)
            {
                //
                // Reset the pending device number.
                //
                g_ucDevNumPending = 0;

                //
                // Set the configuration of the message objects.
                //
                CANDeviceNumSet(0);
            }

            //
            // Indicate that the CAN controller has left assignment mode.
            //
            LEDAssignStop();
        }
    }
    //
    // Enumeration state is handled in this case.
    //
    else if(eCANState == STATE_ENUMERATE)
    {
        //
        // Wait for the correct tick time.
        //
        if(g_ulEventTick == g_ulTickCount)
        {
            //
            // Return to the idle state.
            //
            eCANState = STATE_IDLE;

            //
            // Send out the enumeration data.
            //
            CANSendBroadcastMsg(
                CAN_MSGID_API_ENUMERATE | g_sParameters.ucDeviceNumber, 0, 0);
        }
    }
}
