//*****************************************************************************
//
// controller.c - The motor controller.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#include "../../../hw_ints.h"
#include "../../../hw_memmap.h"
#include "../../../hw_types.h"
#include "../../../driverlib/adc.h"
#include "../../../driverlib/interrupt.h"
#include "../../../driverlib/gpio.h"
#include "../../../driverlib/pwm.h"
#include "../../../driverlib/rom.h"
#include "../../../driverlib/sysctl.h"
#include "../../../driverlib/systick.h"
#include "adc_ctrl.h"
#include "button.h"
#include "can_if.h"
#include "commands.h"
#include "constants.h"
#include "controller.h"
#include "encoder.h"
#include "fan.h"
#include "hbridge.h"
#include "led.h"
#include "limit.h"
#include "math.h"
#include "param.h"
#include "pid.h"
#include "servo_if.h"
#include "wdog.h"

//*****************************************************************************
//
// A count down time for how long after a fault occurs before the fault
// condition is cleared and the controller reset.
//
//*****************************************************************************
unsigned long g_ulFaultCounter = 0;

//*****************************************************************************
//
// The current drive voltage.  This is determined by the current control mode
// (voltage, current, or speed).
//
//*****************************************************************************
static long g_lVoltage = 0;

//*****************************************************************************
//
// The target voltage for voltage control mode.
//
//*****************************************************************************
static long g_lVoltageTarget = 0;

//*****************************************************************************
//
// The rate at which the voltage is adjusted from the current value to the
// target value.  This is defined as the number of steps per millisecond.
//
//*****************************************************************************
static unsigned long g_ulVoltageRate = 0;

//*****************************************************************************
//
// The target speed for speed control mode.
//
//*****************************************************************************
static long g_lSpeedTarget = 0;

//*****************************************************************************
//
// The PID controller used in speed control mode.
//
//*****************************************************************************
static tPIDState g_sSpeedPID;

//*****************************************************************************
//
// The target encoder position in encoder position mode.
//
//*****************************************************************************
static long g_lPositionTarget = 0;

//*****************************************************************************
//
// The PID controller used in encoder position mode.
//
//*****************************************************************************
static tPIDState g_sPositionPID;

//*****************************************************************************
//
// The target current for current control mode.
//
//*****************************************************************************
static long g_lCurrentTarget = 0;

//*****************************************************************************
//
// The PID controller used in current control mode.
//
//*****************************************************************************
static tPIDState g_sCurrentPID;

//*****************************************************************************
//
// A set of flags that indicate the current status of the motor controller.
//
//*****************************************************************************
#define FLAG_FAULT              0
#define FLAG_LOST_LINK          1
#define FLAG_HAVE_LINK          2
#define FLAG_SERVO_LINK         3
#define FLAG_CAN_LINK           4
#define FLAG_CURRENT_MODE       5
#define FLAG_SPEED_MODE         6
#define FLAG_POSITION_MODE      7
#define FLAG_SPEED_SRC_ENCODER  8
#define FLAG_POS_SRC_ENCODER    9
#define FLAG_POS_SRC_POT        10
static unsigned long g_ulFlags = 0;

//*****************************************************************************
//
// A set of flags that indicate the reason for the current fault condition.
//
//*****************************************************************************
static unsigned long g_ulFaultFlags = 0;

//*****************************************************************************
//
// The current state of the controller.
//
//*****************************************************************************
#define STATE_WAIT_FOR_LINK     0
#define STATE_RUN               1
#define STATE_FAULT             2
static unsigned long g_ulControllerState = STATE_WAIT_FOR_LINK;

//*****************************************************************************
//
// This function indicates that the control link is good.
//
//*****************************************************************************
void
ControllerLinkGood(unsigned long ulType)
{
    //
    // See if the control link is CAN.
    //
    if(ulType == LINK_TYPE_CAN)
    {
        //
        // Indicate that the control link is CAN.
        //
        HWREGBITW(&g_ulFlags, FLAG_SERVO_LINK) = 0;
        HWREGBITW(&g_ulFlags, FLAG_CAN_LINK) = 1;
    }
    else
    {
        //
        // Indicate that the control link is servo.
        //
        HWREGBITW(&g_ulFlags, FLAG_CAN_LINK) = 0;
        HWREGBITW(&g_ulFlags, FLAG_SERVO_LINK) = 1;
    }

    //
    // Indicate that the control link is good.
    //
    HWREGBITW(&g_ulFlags, FLAG_HAVE_LINK) = 1;
}

//*****************************************************************************
//
// This function indicates that the control link has been lost.
//
//*****************************************************************************
void
ControllerLinkLost(void)
{
    //
    // Clear the link type.
    //
    HWREGBITW(&g_ulFlags, FLAG_SERVO_LINK) = 0;
    HWREGBITW(&g_ulFlags, FLAG_CAN_LINK) = 0;

    //
    // Indicate that the control link has been lost.
    //
    HWREGBITW(&g_ulFlags, FLAG_LOST_LINK) = 1;
}

//*****************************************************************************
//
// This function returns the type of control link that is in use.
//
//*****************************************************************************
unsigned long
ControllerLinkType(void)
{
    //
    // Determine the type of control link (if any) and return it.
    //
    if(HWREGBITW(&g_ulFlags, FLAG_SERVO_LINK) == 1)
    {
        return(LINK_TYPE_SERVO);
    }
    else if(HWREGBITW(&g_ulFlags, FLAG_CAN_LINK) == 1)
    {
        return(LINK_TYPE_CAN);
    }
    else
    {
        return(LINK_TYPE_NONE);
    }
}

//*****************************************************************************
//
// This function is used to indicate that a fault has occurred.
//
//*****************************************************************************
void
ControllerFaultSignal(unsigned long ulFault)
{
    //
    // Save the new fault in the fault flags.
    //
    g_ulFaultFlags |= ulFault;

    //
    // Indicate that there has been a fault.
    //
    HWREGBITW(&g_ulFlags, FLAG_FAULT) = 1;
}

//*****************************************************************************
//
// The function returns the set of faults that are currently active (if any).
//
//*****************************************************************************
unsigned long
ControllerFaultsActive(void)
{
    //
    // Return the set of currently active faults.
    //
    return(g_ulFaultFlags);
}

//*****************************************************************************
//
// This function returns the current output voltage.
//
//*****************************************************************************
long
ControllerVoltageGet(void)
{
    //
    // Return the current output voltage.
    //
    return(g_lVoltage);
}

//*****************************************************************************
//
// This function is used to force the motor controller into neutral.
//
//*****************************************************************************
static void
ControllerForceNeutral(void)
{
    //
    // Reset the current voltage to neutral.
    //
    g_lVoltage = 0;
    HBridgeVoltageSet(0);

    //
    // Reset the target voltage to neutral.
    //
    g_lVoltageTarget = 0;

    //
    // Reset the speed target to zero and reset the PID controller.
    //
    g_lSpeedTarget = 0;
    PIDReset(&g_sSpeedPID);

    //
    // Reset the position PID controller.
    //
    g_lPositionTarget = ControllerPositionGet();
    PIDReset(&g_sPositionPID);

    //
    // Reset the current target to zero and reset the PID controller.
    //
    g_lCurrentTarget = 0;
    PIDReset(&g_sCurrentPID);
}

//*****************************************************************************
//
// This function is used to enable or disable voltage control mode.
//
//*****************************************************************************
void
ControllerVoltageModeSet(unsigned long bEnable)
{
    //
    // Force the controller into neutral.
    //
    ControllerForceNeutral();

    //
    // See if voltage control mode is being enabled.
    //
    if(bEnable)
    {
        //
        // Disable the other control modes, which enables voltage control mode
        // since it is the default.
        //
        HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE) = 0;
    }
}

//*****************************************************************************
//
// This function is used to set the target output voltage when in voltage
// control mode.
//
//*****************************************************************************
void
ControllerVoltageSet(long lVoltage)
{
    //
    // If the voltage is less than the start of the reverse voltage plateau,
    // then set the voltage to full reverse.
    //
    if(lVoltage < (-32768 + REVERSE_PLATEAU))
    {
        lVoltage = -32768;
    }

    //
    // If the voltage is within the neutral voltage plateau, then set the
    // voltage to neutral.
    //
    if((lVoltage >= (0 - NEUTRAL_PLATEAU)) &&
       (lVoltage <= NEUTRAL_PLATEAU))
    {
        lVoltage = 0;
    }

    //
    // If the voltage is greater than the start of the forward voltage plateau,
    // then set the voltage to full forward.
    //
    if(lVoltage >= (32767 - FORWARD_PLATEAU))
    {
        lVoltage = 32767;
    }

    //
    // Save the target voltage.
    //
    g_lVoltageTarget = lVoltage;
}

//*****************************************************************************
//
// This function sets the rate of change of the output voltage when in voltage
// control mode.
//
//*****************************************************************************
void
ControllerVoltageRateSet(unsigned long ulRate)
{
    //
    // Save the voltage change rate.
    //
    g_ulVoltageRate = ulRate;
}

//*****************************************************************************
//
// This function is used to enable or disable speed control mode.
//
//*****************************************************************************
void
ControllerSpeedModeSet(unsigned long bEnable)
{
    //
    // Force the controller into neutral.
    //
    ControllerForceNeutral();

    //
    // See if speed mode is being enabled.
    //
    if(bEnable)
    {
        //
        // Enable speed mode and disable the other control modes.
        //
        HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE) = 1;
        HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE) = 0;
    }
    else
    {
        //
        // Disable speed mode.
        //
        HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE) = 0;
    }
}

//*****************************************************************************
//
// This function is used to set the target speed when in speed control mode.
//
//*****************************************************************************
void
ControllerSpeedSet(long lSpeed)
{
    //
    // Save the target speed.
    //
    g_lSpeedTarget = lSpeed;
}

//*****************************************************************************
//
// This function is used to get the current speed of the motor.
//
//*****************************************************************************
long
ControllerSpeedGet(void)
{
    //
    // Get the current speed from the currently configured speed sense source.
    //
    if(HWREGBITW(&g_ulFlags, FLAG_SPEED_SRC_ENCODER) == 1)
    {
        return(EncoderVelocityGet());
    }
    else
    {
        return(0);
    }
}

//*****************************************************************************
//
// This function is used to set the speed sense source.
//
//*****************************************************************************
void
ControllerSpeedSrcSet(unsigned long ulSrc)
{
    //
    // Set the speed sense source flag based on the specified speed sense
    // source.
    //
    if(ulSrc == SPEED_SRC_ENCODER)
    {
        HWREGBITW(&g_ulFlags, FLAG_SPEED_SRC_ENCODER) = 1;
    }
    else
    {
        HWREGBITW(&g_ulFlags, FLAG_SPEED_SRC_ENCODER) = 0;
    }
}

//*****************************************************************************
//
// This function sets the P gain of the PID controller for speed control mode.
//
//*****************************************************************************
void
ControllerSpeedPGainSet(long lPGain)
{
    //
    // Set the P gain on the speed PID controller.
    //
    PIDGainPSet(&g_sSpeedPID, lPGain);
}

//*****************************************************************************
//
// This function sets the I gain of the PID controller for speed control mode.
//
//*****************************************************************************
void
ControllerSpeedIGainSet(long lIGain)
{
    long lTemp;

    //
    // Set the I gain and readjust the integrator min/max on the speed PID
    // controller.
    //
    lTemp = MathDiv16x16(32767 * 256, lIGain);
    if(lTemp < 0)
    {
        lTemp = 0 - lTemp;
    }
    PIDGainISet(&g_sSpeedPID, lIGain, lTemp, 0 - lTemp);
}

//*****************************************************************************
//
// This function sets the D gain of the PID controller for speed control mode.
//
//*****************************************************************************
void
ControllerSpeedDGainSet(long lDGain)
{
    //
    // Set the D gain on the speed PID controller.
    //
    PIDGainDSet(&g_sSpeedPID, lDGain);
}

//*****************************************************************************
//
// This function is used to enable or disable position control mode.
//
//*****************************************************************************
void
ControllerPositionModeSet(unsigned long bEnable, long lStartingPosition)
{
    //
    // Force the controller into neutral.
    //
    ControllerForceNeutral();

    //
    // See if position mode is being enabled.
    //
    if(bEnable)
    {
        //
        // Enable position control mode and disable the other control modes.
        //
        HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE) = 1;

        //
        // Provide the ``start'' position to the encoder interface.
        //
        EncoderPositionSet(lStartingPosition);

        //
        // Set the default target position to the current position, based on
        // the currently selected position sense source.
        //
        if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) == 1)
        {
            g_lPositionTarget = lStartingPosition;
        }
        else if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) == 1)
        {
            g_lPositionTarget = ADCPotPosGet();
        }
        else
        {
            g_lPositionTarget = 0;
        }
    }
    else
    {
        //
        // Disable position mode.
        //
        HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE) = 0;
    }
}

//*****************************************************************************
//
// This function is used to set the target position when in position control
// mode.
//
//*****************************************************************************
void
ControllerPositionSet(long lPosition)
{
    //
    // Save the target encoder position.
    //
    g_lPositionTarget = lPosition;
}

//*****************************************************************************
//
// This function is used to get the current position of the motor.
//
//*****************************************************************************
long
ControllerPositionGet(void)
{
    //
    // Get the current position from the currently configured position sense
    // source.
    //
    if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) == 1)
    {
        return(EncoderPositionGet());
    }
    else if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) == 1)
    {
        return(ADCPotPosGet());
    }
    else
    {
        return(0);
    }
}

//*****************************************************************************
//
// This function is used to set the position sense source.
//
//*****************************************************************************
void
ControllerPositionSrcSet(unsigned long ulSrc)
{
    //
    // Set the position sense source flag based on the specified position sense
    // source.
    //
    if(ulSrc == POSITION_SRC_ENCODER)
    {
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) = 1;
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) = 0;
    }
    else if(ulSrc == POSITION_SRC_POT)
    {
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) = 0;
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) = 1;
    }
    else
    {
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) = 0;
        HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) = 0;
    }
}

//*****************************************************************************
//
// This function sets the P gain of the PID controller for position control
// mode.
//
//*****************************************************************************
void
ControllerPositionPGainSet(long lPGain)
{
    //
    // Set the P gain on the encoder position PID controller.
    //
    PIDGainPSet(&g_sPositionPID, lPGain);
}

//*****************************************************************************
//
// This function sets the I gain of the PID controller for position control
// mode.
//
//*****************************************************************************
void
ControllerPositionIGainSet(long lIGain)
{
    long lTemp;

    //
    // Set the I gain and readjust the integrator min/max on the position PID
    // controller.
    //
    lTemp = MathDiv16x16(32767 * 256, lIGain);
    if(lTemp < 0)
    {
        lTemp = 0 - lTemp;
    }
    PIDGainISet(&g_sPositionPID, lIGain, lTemp, 0 - lTemp);
}

//*****************************************************************************
//
// This function sets the D gain of the PID controller for position control
// mode.
//
//*****************************************************************************
void
ControllerPositionDGainSet(long lDGain)
{
    //
    // Set the D gain on the position PID controller.
    //
    PIDGainDSet(&g_sPositionPID, lDGain);
}

//*****************************************************************************
//
// This function is used to enable or disable current control mode.
//
//*****************************************************************************
void
ControllerCurrentModeSet(unsigned long bEnable)
{
    //
    // Force the controller into neutral.
    //
    ControllerForceNeutral();

    //
    // See if current control mode is being enabled.
    //
    if(bEnable)
    {
        //
        // Enable current control mode and disable the other control modes.
        //
        HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE) = 1;
        HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE) = 0;
        HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE) = 0;
    }
    else
    {
        //
        // Disable current control mode.
        //
        HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE) = 0;
    }
}

//*****************************************************************************
//
// This function is used to set the target current when in current control
// mode.
//
//*****************************************************************************
void
ControllerCurrentSet(long lCurrent)
{
    //
    // Save the target current.
    //
    g_lCurrentTarget = lCurrent;
}

//*****************************************************************************
//
// This function sets the P gain of the PID controller for current control
// mode.
//
//*****************************************************************************
void
ControllerCurrentPGainSet(long lPGain)
{
    //
    // Set the P gain on the current PID controller.
    //
    PIDGainPSet(&g_sCurrentPID, lPGain);
}

//*****************************************************************************
//
// This function sets the I gain of the PID controller for current control
// mode.
//
//*****************************************************************************
void
ControllerCurrentIGainSet(long lIGain)
{
    long lTemp;

    //
    // Set the I gain and readjust the integrator min/max on the current PID
    // controller.
    //
    lTemp = MathDiv16x16(32767 * 256, lIGain);
    if(lTemp < 0)
    {
        lTemp = 0 - lTemp;
    }
    PIDGainISet(&g_sCurrentPID, lIGain, lTemp, 0 - lTemp);
}

//*****************************************************************************
//
// This function sets the D gain of the PID controller for current control
// mode.
//
//*****************************************************************************
void
ControllerCurrentDGainSet(long lDGain)
{
    //
    // Set the D gain on the current PID controller.
    //
    PIDGainDSet(&g_sCurrentPID, lDGain);
}

//*****************************************************************************
//
// This function handles the periodic processing for voltage control mode.
//
//*****************************************************************************
static void
ControllerVoltageMode(void)
{
    //
    // There is only something to be done if the target voltage does not match
    // the current output voltage.
    //
    if(g_lVoltage != g_lVoltageTarget)
    {
        //
        // If output voltage ramping is disabled or the target voltage is
        // neutral, then go directly to the target voltage.
        //
        if((g_ulVoltageRate == 0) ||
           ((g_lVoltageTarget >= (0 - NEUTRAL_PLATEAU)) &&
            (g_lVoltageTarget <= NEUTRAL_PLATEAU)))
        {
            g_lVoltage = g_lVoltageTarget;
        }

        //
        // See if the current output voltage is less than the target voltage.
        //
        else if(g_lVoltage < g_lVoltageTarget)
        {
            //
            // Increment the output voltage by the ramp rate and do not allow
            // it to exceed the target voltage.
            //
            g_lVoltage += g_ulVoltageRate;
            if(g_lVoltage > g_lVoltageTarget)
            {
                g_lVoltage = g_lVoltageTarget;
            }
        }
        else
        {
            //
            // Decrement the output voltage by the ramp rate and do not allow
            // it to exceed the target voltage.
            //
            g_lVoltage -= g_ulVoltageRate;
            if(g_lVoltage < g_lVoltageTarget)
            {
                g_lVoltage = g_lVoltageTarget;
            }
        }

        //
        // Send the new output voltage to the H-bridge.
        //
        HBridgeVoltageSet(g_lVoltage);
    }
}

//*****************************************************************************
//
// This function handles the periodic processing for current control mode.
//
//*****************************************************************************
static void
ControllerCurrentMode(void)
{
    long lTemp;

    //
    // If the target current is zero, then the output voltage is zero also.
    //
    if(g_lCurrentTarget == 0)
    {
        g_lVoltage = 0;
    }
    else
    {
        //
        // Get the winding current.
        //
        lTemp = ADCCurrentGet();
        if(g_lVoltage < 0)
        {
            lTemp = 0 - lTemp;
        }

        //
        // Compute the error between the target current and the winding
        // current.
        //
        lTemp = g_lCurrentTarget - lTemp;

        //
        // Run the PID controller, with the output being the output voltage
        // that should be driven to the motor.
        //
        lTemp = PIDUpdate(&g_sCurrentPID, lTemp * 256) / 256;

        //
        // Limit the output voltage to the valid values.
        //
        if(lTemp < -32768)
        {
            lTemp = -32768;
        }
        if(lTemp > 32767)
        {
            lTemp = 32767;
        }

        //
        // Do not drive the motor if the desired output voltage is the opposite
        // polarity of the target current (in other words, do not drive the
        // motor forward if the target current is reverse), or if the desired
        // output voltage is within the neutral plateau.
        //
        if(((g_lCurrentTarget < 0) && (lTemp > 0)) ||
           ((g_lCurrentTarget > 0) && (lTemp < 0)) ||
           ((lTemp >= (0 - NEUTRAL_PLATEAU)) && (lTemp <= NEUTRAL_PLATEAU)))
        {
            lTemp = 0;
        }

        //
        // Set the new output voltage.
        //
        g_lVoltage = lTemp;
    }

    //
    // Send the new output voltage to the H-bridge.
    //
    HBridgeVoltageSet(g_lVoltage);
}

//*****************************************************************************
//
// This function handles the periodic processing for speed control mode.
//
//*****************************************************************************
static void
ControllerSpeedMode(void)
{
    long lTemp;

    //
    // If the target speed is zero, then the output voltage is zero also.
    //
    if(g_lSpeedTarget == 0)
    {
        g_lVoltage = 0;
    }
    else
    {
        //
        // Get the motor speed.
        //
        if(HWREGBITW(&g_ulFlags, FLAG_SPEED_SRC_ENCODER) == 1)
        {
            lTemp = EncoderVelocityGet();
        }
        else
        {
            lTemp = 0;
        }
        if(g_lVoltage < 0)
        {
            lTemp = 0 - lTemp;
        }

        //
        // Compute the error between the target speed and the motor speed.
        //
        lTemp = g_lSpeedTarget - lTemp;

        //
        // Run the PID controller, with the output being the output voltage
        // that should be driven to the motor.
        //
        lTemp = PIDUpdate(&g_sSpeedPID, lTemp) / 256;

        //
        // Limit the output voltage to the valid values.
        //
        if(lTemp < -32768)
        {
            lTemp = -32768;
        }
        if(lTemp > 32767)
        {
            lTemp = 32767;
        }

        //
        // Do not drive the motor if the desired output voltage is the opposite
        // polarity of the target speed (in other words, do not drive the motor
        // forward if the target speed is reverse), or if the desired output
        // voltage is within the neutral plateau.
        //
        if(((g_lSpeedTarget < 0) && (lTemp > 0)) ||
           ((g_lSpeedTarget > 0) && (lTemp < 0)) ||
           ((lTemp >= (0 - NEUTRAL_PLATEAU)) && (lTemp <= NEUTRAL_PLATEAU)))
        {
            lTemp = 0;
        }

        //
        // Set the new output voltage.
        //
        g_lVoltage = lTemp;
    }

    //
    // Send the new output voltage to the H-bridge.
    //
    HBridgeVoltageSet(g_lVoltage);
}

//*****************************************************************************
//
// This function handles the periodic processing for position control mode.
//
//*****************************************************************************
static void
ControllerPositionMode(void)
{
    long lTemp;

    //
    // Get the motor position.
    //
    if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_ENCODER) == 1)
    {
        lTemp = EncoderPositionGet();
    }
    else if(HWREGBITW(&g_ulFlags, FLAG_POS_SRC_POT) == 1)
    {
        lTemp = ADCPotPosGet();
    }
    else
    {
        lTemp = 0;
    }

    //
    // Compute the error between the target position and the motor position.
    //
    lTemp = g_lPositionTarget - lTemp;

    //
    // Run the PID controller, with the output being the output voltage that
    // should be driven to the motor.
    //
    lTemp = PIDUpdate(&g_sPositionPID, lTemp) / 256;

    //
    // Limit the output voltage to the valid values.
    //
    if(lTemp < -32768)
    {
        lTemp = -32768;
    }
    if(lTemp > 32767)
    {
        lTemp = 32767;
    }

    //
    // Do not drive the motor if the desired output voltage is within the
    // neutral plateau.
    //
    if((lTemp >= (0 - NEUTRAL_PLATEAU)) && (lTemp <= NEUTRAL_PLATEAU))
    {
        lTemp = 0;
    }

    //
    // Set the new output voltage.
    //
    g_lVoltage = lTemp;

    //
    // Send the new output voltage to the H-bridge.
    //
    HBridgeVoltageSet(g_lVoltage);
}

//*****************************************************************************
//
// This function initializes the controller.
//
//*****************************************************************************
void
ControllerInit(void)
{
    //
    // Initialize the PID controllers.
    //
    PIDInitialize(&g_sCurrentPID, 0, 0, 0, 0, 0);
    PIDInitialize(&g_sSpeedPID, 0, 0, 0, 0, 0);
    PIDInitialize(&g_sPositionPID, 0, 0, 0, 0, 0);
}

//*****************************************************************************
//
// This function is called to handle the timer interrupt from the PWM module,
// which generates the 1 ms system timing.
//
//*****************************************************************************
void
ControllerIntHandler(void)
{
    //
    // Clear the interrupt source.
    //
    ROM_PWMGenIntClear(PWM_BASE, PWM_GEN_2, PWM_INT_CNT_ZERO);

    //
    // Always check the buttons
    //
    ButtonTick();

    //
    // Determine the state of the controller.
    //
    switch(g_ulControllerState)
    {
        //
        // The controller is waiting for a valid link.
        //
        case STATE_WAIT_FOR_LINK:
        {
            //
            // Do not acknowledge a valid link until the ADC calibration is
            // complete.
            //
            if(ADCCalibrationDone() == 0)
            {
                break;
            }

            //
            // If a fault has been signaled, then turn off the fault flag and
            // reset the fault counter.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_FAULT) == 1)
            {
                HWREGBITW(&g_ulFlags, FLAG_FAULT) = 0;
                g_ulFaultCounter = FAULT_TIME;
            }

            //
            // See if the fault counter is active.
            //
            if(g_ulFaultCounter)
            {
                //
                // Decrement the fault counter.
                //
                g_ulFaultCounter--;

                //
                // Clear the fault(s) if the fault counter has reached zero.
                //
                if(g_ulFaultCounter == 0)
                {
                    g_ulFaultFlags = 0;
                }
            }

            //
            // See if a valid link has been detected.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_HAVE_LINK) == 1)
            {
                //
                // Indicate that the link has not been lost.
                //
                HWREGBITW(&g_ulFlags, FLAG_LOST_LINK) = 0;

                //
                // If the fault counter is active, then go to the fault state.
                // Otherwise, go to the run state.
                //
                if(g_ulFaultCounter != 0)
                {
                    g_ulControllerState = STATE_FAULT;
                }
                else
                {
                    g_ulControllerState = STATE_RUN;
                }
            }

            //
            // This state has been handled.
            //
            break;
        }

        //
        // The controller is actively running.
        //
        case STATE_RUN:
        {
            //
            // Process any commands in the queue.
            //
            CommandQueueProcess(0);

            //
            // Call the appropriate control method based on the control mode
            // that is currently enabled.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_CURRENT_MODE))
            {
                ControllerCurrentMode();
            }
            else if(HWREGBITW(&g_ulFlags, FLAG_SPEED_MODE))
            {
                ControllerSpeedMode();
            }
            else if(HWREGBITW(&g_ulFlags, FLAG_POSITION_MODE))
            {
                ControllerPositionMode();
            }
            else
            {
                ControllerVoltageMode();
            }

            //
            // See if the control link has been lost.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_LOST_LINK) == 1)
            {
                //
                // Indicate that the link is not valid.
                //
                HWREGBITW(&g_ulFlags, FLAG_HAVE_LINK) = 0;

                //
                // Abandon any activity that is currently active.
                //
                ControllerForceNeutral();
                ServoIFCalibrationAbort();

                //
                // Wait for the control link to be restored.
                //
                g_ulControllerState = STATE_WAIT_FOR_LINK;
            }

            //
            // See if a fault has occurred.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_FAULT) == 1)
            {
                //
                // Clear the fault flag.
                //
                HWREGBITW(&g_ulFlags, FLAG_FAULT) = 0;

                //
                // Abandon any activity that is currently active.
                //
                ControllerForceNeutral();
                ServoIFCalibrationAbort();

                //
                // Reset the fault counter and go to the fault state.
                //
                g_ulFaultCounter = FAULT_TIME;
                g_ulControllerState = STATE_FAULT;
            }

            //
            // This state has been handled.
            //
            break;
        }

        //
        // The controller is waiting for a fault to clear.
        //
        case STATE_FAULT:
        {
            //
            // If a fault has been signaled, then turn off the fault flag and
            // reset the fault counter.
            //
            if(HWREGBITW(&g_ulFlags, FLAG_FAULT) == 1)
            {
                HWREGBITW(&g_ulFlags, FLAG_FAULT) = 0;
                g_ulFaultCounter = FAULT_TIME;
            }

            //
            // Process any commands in the queue, but ignore any commands that
            // would result in the motor being driven.
            //
            CommandQueueProcess(1);

            //
            // Decrement the fault counter.
            //
            g_ulFaultCounter--;

            //
            // See if the fulat counter has reached zero.
            //
            if(g_ulFaultCounter == 0)
            {
                //
                // Clear the fault(s).
                //
                g_ulFaultFlags = 0;

                //
                // If there is not a control link, then go to the wait for link
                // state.  Otherwise, go to the run state.
                //
                if(HWREGBITW(&g_ulFlags, FLAG_LOST_LINK) == 1)
                {
                    HWREGBITW(&g_ulFlags, FLAG_HAVE_LINK) = 0;
                    g_ulControllerState = STATE_WAIT_FOR_LINK;
                }
                else
                {
                    g_ulControllerState = STATE_RUN;
                }
            }

            //
            // This state has been handled.
            //
            break;
        }
    }

    //
    // Update the limit switches.
    //
    LimitTick();

    //
    // Update the encoder.
    //
    EncoderTick();

    //
    // Update the fan.
    //
    FanTick();

    //
    // Update the LED status
    //
    LEDTick();

    //
    // Call the CAN tick function.
    //
    CANIFTick();
}
