//*****************************************************************************
//
// qs-bdc.c - Brushed DC motor controller application.
//
// Copyright (c) 2008 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
// 
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
// 
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
// 
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
// 
// This is part of revision 3618 of the RDK-BDC Firmware Package.
//
//*****************************************************************************

#include "../../../hw_ints.h"
#include "../../../hw_types.h"
#include "../../../driverlib/interrupt.h"
#include "../../../driverlib/rom.h"
#include "../../../driverlib/sysctl.h"
#include "adc_ctrl.h"
#include "button.h"
#include "can_if.h"
#include "controller.h"
#include "encoder.h"
#include "fan.h"
#include "hbridge.h"
#include "led.h"
#include "limit.h"
#include "param.h"
#include "pins.h"
#include "servo_if.h"
#include "wdog.h"

//*****************************************************************************
//
//! \addtogroup example_list
//! <h1>Brushed DC Motor Controller (qs-bdc)</h1>
//!
//! This application controls a brushed DC motor.  Two communication methods
//! are supported; a hobby servo-style input for basic voltage control or a CAN
//! input for more advanced control (the two inputs are mutually exclusive).
//!
//! When using either communication methods, a basic voltage control mode is
//! available.  In this mode, the external controller directly specifies the
//! desired output voltage.  When using CAN, an output voltage slew rate can be
//! specified, which results in the output voltage adjusting in a linear
//! fashion from the current voltage to the new voltage (as opposed to directly
//! jumping to the new voltage if the slew rate is disabled).
//!
//! Additional advanced control methods are also available when using the CAN
//! communication interface.  There are current control mode, speed control
//! mode, and position control mode.  Each of these modes is mutually exclusive
//! and operating using a PID controller whose gains are fully programmable via
//! the CAN interface.  Each PID controller starts with all of its gains set to
//! zero, so no output voltage will be generated by any of these modes until
//! the PID controller is at least partially configured.
//!
//! In speed control mode, the speed of the motor is measured using the
//! quadrature encoder input.  Only PHA is used for measuring the speed, so it
//! can be used with gear tooth sensors as well (which only provide a single
//! pulse stream, not a quadrature pair as provided by a shaft encoder).
//!
//! In position control mode, the position of the motor can be measured using
//! the quadrature encoder input or the analog input.  When using the analog
//! input, a 10K potentiometer must be coupled to the output shaft of the motor
//! (either before or after gearing) in some manner so that the motor position
//! can be tracked.
//!
//! The status of the motor controller can also be monitored over the CAN
//! interface.  The bus voltage, output voltage, motor current, ambient
//! temperature, speed, position, limit switch values, fault status, power
//! status, and firmware version can all be queried.
//
//*****************************************************************************

//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, unsigned long ulLine)
{
}
#endif

//*****************************************************************************
//
// This example demonstrates how to setup an analog comparator and trigger
// interrupts on output changes.
//
//*****************************************************************************
int
main(void)
{
    //
    // Set the clocking to run from the main oscillator
    //
    ROM_SysCtlClockSet(SYSCTL_SYSDIV_1 | SYSCTL_USE_OSC | SYSCTL_OSC_MAIN |
                       SYSCTL_XTAL_16MHZ);

    //
    // Enable the peripherals used by this application.
    //
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_CAN0);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_PWM);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_QEI0);
    ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_WDOG);

    //
    // Set the priorities of the interrupts.  The following interrupts are used
    // (listed in priority from highest to lowest):
    //
    //     Watchdog    - Detects fault conditions due to a lack of input
    //                   control signal (servo or CAN).  This is the highest
    //                   priority since its occurrence should be immediately
    //                   handled, regardless of whatever else may be happening.
    //
    //     Servo input - The PWM input control signal.  This is next highest
    //                   priority since delays in handling this interrupt will
    //                   result in "jitter" in the detected input pulse width.
    //
    //     QEI         - The input from the quadrature encoder.  This is the
    //                   same priority as the servo input since CAN must be
    //                   used in order to use the encoder (in other words, QEI
    //                   and servo are mutually exclusive), and delays in
    //                   handling the QEI input interrupt will result in
    //                   "jitter" in the measured speed.
    //
    //     ADC0        - The completion of an ADC sampling.
    //
    //     PWM2        - The periodic control loop for the application.
    //
    //     CAN0        - The CAN control traffic.
    //
    ROM_IntPrioritySet(INT_WATCHDOG, 0x00);
    ROM_IntPrioritySet(SERVO_INT, 0x20);
    ROM_IntPrioritySet(QEI_PHA_INT, 0x20);
    ROM_IntPrioritySet(INT_ADC0, 0x40);
    ROM_IntPrioritySet(INT_PWM2, 0x60);
    ROM_IntPrioritySet(INT_CAN0, 0x80);

    //
    // Initialize the controller.
    //
    ControllerInit();

    //
    // Enable the LED interface
    //
    LEDInit();

    //
    // Get the initial configuration from flash if present.
    //
    ParamInit();

    //
    // Enable the button
    //
    ButtonInit();

    //
    // Enable the limit switch inputs
    //
    LimitInit();

    //
    // Initialize the fan
    //
    FanInit();

    //
    // Configure the ADC
    //
    ADCInit();

    //
    // Enable the H-Bridge interface
    //
    HBridgeInit();

    //
    // Enable the Servo (PWM) interface
    //
    ServoIFInit();

    //
    // Configure the encoder interface.
    //
    EncoderInit();

    //
    // Configure and enable the watchdog interrupt
    //
    WatchdogInit();

    //
    // Initial configuration of the CAN network interface.
    //
    CANIFInit();

    //
    // Enable processor interrupts
    //
    IntMasterEnable();

    //
    // Loop forever.  All the work is done in interrupt handlers, so there is
    // nothing to be done here.  If there is background work to be done with
    // any processor cycles left over after controlling the motor, then it can
    // be placed here.
    //
    while(1)
    {
        //
        // Put the processor to sleep until the next interrupt.
        //
#ifndef DEBUG
        SysCtlSleep();
#endif
    }
}
